;;;; cgi-handler.scm
;
; Copyright (c) 2007, Peter Bex
; Copyright (c) 2000-2005, Felix L. Winkelmann
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.
;
; CGI file handler
; See the spec at http://hoohoo.ncsa.uiuc.edu/cgi/interface.html

(declare 
 (export cgi-handler cgi-handler* spiffy-cgi-default-environment) )

(define (cgi-handler* interp)
  (lambda (fn) (cgi-handler fn interp)))

(define (alist->envlist alist)
  (map (lambda (entry)
	 (string-append (car entry) "=" (cdr entry)))
       alist))

(define (cgi-build-env req fn)
  (let* ((attrs (http:request-attributes req))
	 (url (http:request-url req))
	 (m (string-match "([^?]+)\\?(.*)" url))
	 (server-env `(("SERVER_PROTOCOL" . ,(->string (http:request-protocol req)))
		       ("SERVER_PORT" . ,(->string (spiffy-tcp-port)))
		       ("REQUEST_METHOD" . ,(->string (http:request-method req)))
		       ("CONTENT_TYPE" . ,(alist-ref "content-type" attrs string=? "application/octet-stream"))
		       ("CONTENT_LENGTH" . ,(alist-ref "content-length" attrs string=?
						       (->string (string-length (http:request-unparsed-body req)))))
		       ("SCRIPT_NAME" . ,(if m (cadr m) url))
		       ("SCRIPT_FILENAME" . ,fn)
		       ("REMOTE_ADDR" . ,(http:request-ip req))
		       ("QUERY_STRING" . ,(if m (caddr m) ""))
		       ("SERVER_NAME" . ,(current-hostname))
		       ("PATH_INFO" . ,(or (current-pathinfo) ""))
		       ("PATH_TRANSLATED" . "")))
	 (headers-env (map (lambda (attr)
			     (cons (string-append "HTTP_" (string-translate (string-upcase (car attr)) "-" "_"))
				   (cdr attr)))
			   attrs)))
    (alist->envlist (append (spiffy-cgi-default-environment) headers-env server-env))))

(define (cgi-handler fn . args)
  (let* ((req (current-request))
	 (interp (:optional args (string-match "([^?]+)\\?(.*)" (http:request-url req))))
	 (body (http:request-unparsed-body req))
	 (env (cgi-build-env req fn)))
    (if (file-execute-access? interp)
	(let-values (((i o pid) (process interp (list interp fn) env)))
		    (spiffy-debug "(cgi) started program ~a(~a) ..." interp fn)
		    (display body o)
		    (close-output-port o)
		    (let loop ()
		      (let ((ln (read-line i)))
			(cond ((eof-object? ln)
			       (http:write-error-response 500 "Internal server error") )
			      ((string=? "" ln)
			       (let* ((body (read-string #f i))
				      (len (string-length body)))
				 (close-input-port i)
				 (spiffy-debug "(cgi) program terminated normally, response has ~a bytes" len)
				 (set-header! (sprintf "Content-Length: ~A") len)
				 (write-response-header)
				 (unless (eq? 'HEAD (http:request-method req))
					 (display body) ) ) )
			      ((string-match "Location:[ \t]([^ \t].+)" ln) =>
			       (lambda (m)
				 (redirect (cadr m))
				 (loop) ) )
			      (else
			       (set-header! ln)
			       (loop) ) ) ) ) )
	(http:write-error-response 500 "Internal server error") ) ) )

(define spiffy-cgi-default-environment
  (make-parameter `(("SERVER_SOFTWARE" . ,(spiffy-server-name))
		    ("GATEWAY_INTERFACE" . "CGI/1.1")
		    ("REDIRECT_STATUS" . "200"))))
