;;;; simple-directory-handler.scm
;
; Copyright (c) 2007, Peter Bex
; Copyright (c) 2000-2005, Felix L. Winkelmann
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

(declare
 (export simple-directory-handler simple-directory-css simple-directory-dotfiles? simple-directory-display-file))

(define simple-directory-css (make-parameter
"body {
  background-color: #d6c610;
  font-family: Helvetica, Arial, sans-serif;
}
td {
  padding: 0 1em;
  font-family: Courier, monospace;
}
.directory {
  font-weight: bold;
}
#server-name {
  font-style: italic;
}
.size {
  text-align:right;
}"))

(define simple-directory-dotfiles? (make-parameter #f))
(define simple-directory-display-file
  (make-parameter
   (lambda (remote-file local-file dir?)
     (printf "<tr><td class=\"~a\"><a href=\"~a\">~a</a></td><td class=\"size\">~a</td><td class=\"modification-time\">~a</td></tr>~%"
	     (if dir? "directory" "file")
	     remote-file
	     (htmlize (pathname-strip-directory remote-file))
	     (file-size local-file)
	     (seconds->string (file-modification-time local-file))))))

(define (simple-directory-handler path)
  (let ([str (with-output-to-string
	       (lambda ()
		 (printf "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"><html lang=\"en\"><head><style type=\"text/css\">~a></style><title>Index of ~a</title></head><body><h2>Index of ~a:</h2>
                          <p><a id=\"parent-link\" href=\"~a\">Go to parent directory</a></p><table>"
			 (simple-directory-css)
			 (current-path)
			 (current-path)
			 (pathname-directory (current-path)))
		 (let ([dir (sort (directory path (simple-directory-dotfiles?)) string<?)])
		   (for-each
		    (lambda (file)
		      (let* ((local-file (make-pathname path file))
			     (remote-file (make-pathname (current-path) file))
			     (dir? (directory? local-file)))
			(when (file-exists? local-file)
			      ((simple-directory-display-file) remote-file local-file dir?))))
		    dir)
		   (display "</table><hr />")
		   (printf "<p id=\"server-name\">~A at ~A, port ~A</p></body></html>"
			   (spiffy-server-name) (get-host-name) (spiffy-tcp-port)) ) ) ) ] )
    (set-header! "Content-type: text/html")
    (set-header! (sprintf "Content-length: ~A" (string-length str)))
    (write-response-header)
    (display str)))
