;;;; spiffy.scm
;
; Copyright (c) 2000-2005, Felix L. Winkelmann
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.
;
; Send bugs, suggestions and ideas to: 
;
; felix@call-with-current-continuation.org
;
; Felix L. Winkelmann
; Steinweg 1A
; 37130 Gleichen, OT Weissenborn
; Germany

(define-macro (define-http-resource head . body)
  (let ([url (->string (car head))]
	[rargs (map (lambda (arg)
		      (cond [(and (list? arg) (= 2 (length arg))) 
			     (cons (->string (car arg))
				   (cadr arg)) ]
			    [(pair? arg) (syntax-error 'define-http-resource "invalid argument" arg)]
			    [else (cons (->string arg) #f)] ) )
		    (cdr head) ) ] 
	[req (gensym)]
	[ret (gensym)]
	[args (gensym)] )
    `(http:add-resource
      ,(if (char=? #\/ (string-ref url 0))
	   url
	   (string-append "/" url) )
      (lambda (,req ,args)
	(parameterize ((current-urlencoded-arguments ,args)
		       (current-request ,req)
		       (current-response-headers (list (cons "Content-Type" "text/html")))
		       (current-response-code (cons 200 "OK")))
	  (call/cc
	   (lambda (,ret)
	     (let ([respond (lambda (data)
			      (set-header! (sprintf "Content-Length: ~A" (string-length data)))
			      (write-response-header)
			      (display data)
			      (,ret #f))])
	       (handle-exceptions exn
		 (parameterize ((current-response-code '(500 . "Internal server error")))
			       (emit-error-message exn)
			       ((spiffy-exception-handler) exn)
			       (,ret #f))
		   (respond
		    (let ,(map (lambda (arg)
				 `(,(string->symbol (car arg)) (alist-ref ,(car arg) ,args string-ci=? ,(cdr arg))) )
			       rargs)
		      ,@body)))))))))))