;;;; web-scheme-handler.scm
;
; Copyright (c) 2007, Peter Bex
; Copyright (c) 2000-2005, Felix L. Winkelmann
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
; conditions are met:
;
;   Redistributions of source code must retain the above copyright notice, this list of conditions and the following
;     disclaimer. 
;   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
;     disclaimer in the documentation and/or other materials provided with the distribution. 
;   Neither the name of the author nor the names of its contributors may be used to endorse or promote
;     products derived from this software without specific prior written permission. 
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
; OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
; AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.
;
; Web-scheme file handler

(declare 
  (export web-scheme-handler web-scheme-eval-environment) )

(define web-scheme-eval-environment (make-parameter (interaction-environment)))

(define-macro (swallow-output . exprs)
  (let ((ret (gensym))
	(str (gensym)))
    `(let* ((,ret "")
	    (,str (with-output-to-string
		    (lambda () (set! ,ret (begin ,@exprs))))))
       (if (string? ,ret)
	   (string-append ,str (->string ,ret))
	   ,str))))

(define (load-ws file)
  (swallow-output (eval `(begin ,@(read-file file)) (web-scheme-eval-environment))))

(define (web-scheme-handler fn)
  (let* ([out (load-ws fn)])
    (set-header! (sprintf "Content-Length: ~A" (string-length out)))
    (write-response-header)
    (unless (eq? 'HEAD (http:request-method (current-request)))
      (display out) ) ) )

